@@ -74,7 +74,11 @@ class TelepathyPlugin(gobject.GObject):
             # We were invited to join a chat or a media call
             # args:
             #   channel object path
-            (gobject.SIGNAL_RUN_FIRST, None, [object]),
+            (gobject.SIGNAL_RUN_FIRST, None, [object, str]),
+        'want-to-connect':
+            # The TelepathyPlugin wants to connect. presenceservice.py will
+            # call the start() method if that's OK with its policy.
+            (gobject.SIGNAL_RUN_FIRST, None, []),
     }
 
     _RECONNECT_INITIAL_TIMEOUT = 5000    # 5 seconds
@@ -163,9 +167,6 @@ class TelepathyPlugin(gobject.GObject):
         """
         raise NotImplementedError
 
-    def start(self):
-        raise NotImplementedError
-
     def suggest_room_for_activity(self, activity_id):
         """Suggest a room to use to share the given activity.
         """
@@ -178,6 +179,7 @@ class TelepathyPlugin(gobject.GObject):
         """Attempt to reconnect to the server after the back-off time has
         elapsed.
         """
+        _logger.debug("%r: reconnect timed out. Let's try to connect", self)
         if self._backoff_id > 0:
             gobject.source_remove(self._backoff_id)
             self._backoff_id = 0
@@ -191,6 +193,8 @@ class TelepathyPlugin(gobject.GObject):
         if self._backoff_id != 0:
             gobject.source_remove(self._backoff_id)
 
+        _logger.debug("%r: restart reconnect time out (%u seconds)",
+                self, self._reconnect_timeout / 1000)
         self._backoff_id = gobject.timeout_add(self._reconnect_timeout,
                 self._reconnect_cb)
 
@@ -208,9 +212,13 @@ class TelepathyPlugin(gobject.GObject):
         if there is an existing connection, reuse it by
         registering for various of events on it.
         """
+        _logger.debug('%r: init connection', self)
         conn = self._find_existing_connection()
         if not conn:
+            _logger.debug('%r: no existing connection. Create a new one', self)
             conn = self._make_new_connection()
+        else:
+            _logger.debug('%r: found existing connection. Reuse it', self)
 
         m = conn[CONN_INTERFACE].connect_to_signal('StatusChanged',
            self._handle_connection_status_change)
@@ -285,14 +293,14 @@ class TelepathyPlugin(gobject.GObject):
             _logger.debug("%r: disconnected (reason %r)", self, reason)
             if reason == CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED:
                 # FIXME: handle connection failure; retry later?
+                _logger.debug("%r: authentification failed. Give up ", self)
                 pass
             else:
                 # Try again later. We'll detect whether we have a network
                 # connection after the retry period elapses. The fact that
                 # this timer is running also serves as a marker to indicate
                 # that we shouldn't try to go back online yet.
-                if self._backoff_id:
-                    self._reset_reconnect_timer()
+                self._reset_reconnect_timer()
 
         self.emit('status', self._conn_status, int(reason))
 
@@ -340,6 +348,7 @@ class TelepathyPlugin(gobject.GObject):
             self._backoff_id = 0
 
         self._ip4am.disconnect(self._ip4am_sigid)
+        self._ip4am_sigid = 0
 
     def _contacts_offline(self, handles):
         """Handle contacts going offline (send message, update set)"""
@@ -468,7 +477,7 @@ class TelepathyPlugin(gobject.GObject):
         elif (handle_type == HANDLE_TYPE_CONTACT and
               channel_type in (CHANNEL_TYPE_TEXT,
                                CHANNEL_TYPE_STREAMED_MEDIA)):
-            self.emit("private-invitation", object_path)
+            self.emit("private-invitation", object_path, channel_type)
 
         elif (handle_type == HANDLE_TYPE_LIST and
               channel_type == CHANNEL_TYPE_CONTACT_LIST):
@@ -544,6 +553,11 @@ class TelepathyPlugin(gobject.GObject):
         otherwise initiate a connection and transfer control to
             _connect_reply_cb or _connect_error_cb
         """
+
+        if self._ip4am_sigid == 0:
+            self._ip4am_sigid = self._ip4am.connect('address-changed',
+                    self._ip4_address_changed_cb)
+
         if self._conn is not None:
             return
 
@@ -555,7 +569,7 @@ class TelepathyPlugin(gobject.GObject):
         else:
             _logger.debug('%r: Postponing connection', self)
 
-    def _ip4_address_changed_cb(self, ip4am, address):
+    def _ip4_address_changed_cb(self, ip4am, address, iface):
         _logger.debug("::: IP4 address now %s", address)
 
         self._reconnect_timeout = self._RECONNECT_INITIAL_TIMEOUT
